#!/usr/bin/python
"""
ampcontrol is a basic python interface for communication with
multichannel amplifiers and recievers via the RS-232
communication port.

Copyright (C) 2009  Vidar Waagbo <vidar@waagbo.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""

import sys
from optparse import OptionParser

#PYROTEL CLI APPLICATION

usage = "usage: %prog [options] [device]"
parser = OptionParser(usage=usage)

parser.add_option("-d", "--driver", action="store", dest="driver",
        type="string", metavar="amp",
        help="Select which device driver to use. The default amplifier driver is \"rotel\".")
parser.add_option("-m", "--mute", action="store_true", dest="mute",
		help="Mute amplifier or active zone")
parser.add_option("-M", "--unmute", action="store_false", dest="mute",
		help="Unmute amplifier or active zone")
parser.add_option("-p", "--power-on", action="store_true", dest="power",
		help="Power on the amplifier or selected zone")
parser.add_option("-P", "--power-off", action="store_false", dest="power",
		help="Power on the amplifier or selected zone")
parser.add_option("--force-power-off", action="store_true", dest="force_power_off",
        help="Power off all zones for a complete standby.")
parser.add_option("-s", "--source", action="store", dest="source",
        type="string", metavar="name",
        help="Set main or zone source based on original name without whitespaces. Video channels can be accessed by video[n] or v[n] as source names.")
parser.add_option("-r", "--record", action="store", dest="record",
        type="string", metavar="name",
        help="Set main or zone record source based on original name without whitespaces. Video channels can be accessed by video[n] or v[n] as source names.")
parser.add_option("--toggle-party-mode", action="store_true", dest="toggle_party_mode",
        help="Toggles the party mode setting.")
parser.add_option("-v", "--volume", action="store", 
		type="int", dest="volume", metavar="level",
		help="Set main or zone volume (0-100)")
parser.add_option("--volume-up", dest="volume_up", action="store",
		type="int", metavar="number",
		help="Increase the main or zone volume by n dB")
parser.add_option("--volume-down", dest="volume_down", action="store",
		type="int", metavar="number",
		help="Decrease the main or zone volume by n dB")
parser.add_option("-z", "--zone", action="store", type="int",
		dest="zone", metavar="number",
		help="Select zone to manipulate (1-4), where zone 1 is the main zone.")
parser.add_option("-c", "--cmd", metavar="hex", action="store", dest="command",
        type="string",
        help="Send a custom command to the reciever (advanced users only).")

(options, args) = parser.parse_args()

# Device and driver management. Add more drivers here:
if(args == []):
    device = None
else:
    device = args[0]

if(options.driver):
    driver = options.driver
else:
    driver = "rotel"

if(driver == "rotel"):
    from drivers import rotel
    try:
        control = rotel.Control(device)
    except:
        print("ampcontrol-cli.py: error: invalid serial device specified")
        exit()
elif(driver == "dummy"):
    from drivers import dummy
    try:
        control = dummy.Control(device)
    except:
        print("ampcontrol-cli.py: error: invalid serial device specified")
        exit()
else:
    parser.error("invalid driver")

try:
    if(options.command):
        control.custom_command(options.command)

    # Power management. Main and Zone Power.
    if(options.power != None):
        control.set_power(options.power, options.zone)

    if(options.force_power_off):
        control.complete_power_off()

    # Source management.
    if(options.toggle_party_mode):
        control.toggle_party_mode()

    if(options.source):
        control.set_source(options.source, options.zone)

    if(options.record):
        control.set_record_source(options.record)

    # Volume control. Main and Zone Volume.
    if((options.volume_up and options.volume_down) or
        (options.volume_up and options.volume) or
        (options.volume_down and options.volume)):
        parser.error("options --volume, --volume-up and --volume-down are mutually exclusive")
    if(options.volume_up):
        control.volume_up(options.volume_up, options.zone)
    if(options.volume_down):
        control.volume_down(options.volume_down, options.zone)
    if(options.volume):
        control.set_volume(options.volume, options.zone)
    if(options.mute == True):
        control.mute_on()
    if(options.mute == False):
        control.mute_off()

    control.close_connection()

except Exception as inst:
    parser.error(inst.args[0])

if(len(sys.argv) <= 1):
    parser.print_help()
    exit()
