#!/usr/bin/python
"""
ampcontrol is a basic python interface for communication with
multichannel amplifiers and recievers via the RS-232
communication port.

Copyright (C) 2009  Vidar Waagbo <vidar@waagbo.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

TODO:

    [ ] Total interface control
    [ ] Better secret handling (randomize secret etc.)
    [ ] SSL/encryption support

"""

import sys
from SimpleXMLRPCServer import SimpleXMLRPCServer
from optparse import OptionParser

usage = "usage: %prog [options] [interface][:port]"
parser = OptionParser(usage=usage)

parser.add_option("-d", "--driver", action="store", dest="driver",
        type="string", metavar="amp",
        help="Select which device driver to use. The default amplifier driver is \"rotel\".")
parser.add_option("-e", "--device", action="store", dest="device",
        type="string", metavar="amp",
        help="Select which communication device to use. The default device is \"/dev/ttyUSB0\".")
parser.add_option("-p", "--port", action="store", dest="port",
        type="int", metavar="port",
        help="Select which network port to use.")
parser.add_option("-s", "--secret", action="store", dest="secret",
        type="string", metavar="secret",
        help="Set the secret to use for the XML RPC server. Default secret is 'SECRET' ")

(options, args) = parser.parse_args()

"""
# No arguments at all == quit!
if(len(sys.argv) <= 1):
    parser.print_help()
    exit()
"""

# Server configuration
SimpleXMLRPCServer.allow_reuse_address = 1

if(options.port):
    server = SimpleXMLRPCServer(("", options.port))
else:
    server = SimpleXMLRPCServer(("", 8080))

if(options.secret):
    secret = options.secret
else:
    secret = "SECRET"

print "Secret is:", secret

# Device and driver management. Add more drivers here:
if(args == []):
    device = None
else:
    device = args[0]

if(options.driver):
    driver = options.driver
else:
    driver = "rotel"

if(driver == "rotel"):
    from drivers import rotel
    try:
        control = rotel.Control(device)
    except:
        print("ampcontrol-server.py: error: invalid serial device specified")
        exit()
elif(driver == "dummy"):
    from drivers import dummy
    try:
        control = dummy.Control(device)
    except:
        print("ampcontrol-server.py: error: invalid serial device specified")
        exit()
else:
    parser.error("invalid driver")

class RemoteControl:

    def __init__(self, control, server, secret):
        self.control = control
        self.server = server
        self.secret = secret
        self.server.register_instance(self)
        print "Starting server..."
        self.server.serve_forever()

    def invalid_secret(self, secret):
        if(secret == self.secret):
            return False
        else:
            print secret, self.secret
            return True

    def custom_command(self, secret, command):
        if self.invalid_secret(secret):
            return "FORBIDDEN"
        try:
            self.control.custom_command(command)
            return "OK"     
        except:
            return "FAIL"

    def toggle_party_mode(self, secret):
        if self.invalid_secret(secret):
            return "FORBIDDEN"
        try:
            self.control.toggle_party_mode()
            return "OK"
        except:
            return "FAIL"

    def set_source(self, secret, source, zone = None):
        if self.invalid_secret(secret):
            return "FORBIDDEN"
        try:
            self.control.set_source(source, zone)
            return "OK"
        except:
            return "FAIL"

    def set_record_source(self, secret, source):
        if self.invalid_secret(secret):
            return "FORBIDDEN"
        try:
            self.control.set_record_source(source)
            return "OK"
        except:
            return "FAIL"

    def set_power(self, secret, state, zone = None):
        if self.invalid_secret(secret):
            return "FORBIDDEN"
        try:
            self.control.set_power(state, zone)
            return "OK"
        except:
            return "FAIL"

    def complete_power_off(self, secret):
        if self.invalid_secret(secret):
            return "FORBIDDEN"
        try:
            self.control.complete_power_off()
            return "OK"
        except:
            return "FAIL"

    def set_volume(self, secret, amount, zone = None):
        if self.invalid_secret(secret):
            return "FORBIDDEN"
        try:
            self.control.set_volume(amount, zone)
            return "OK"
        except:
            return "FAIL"

    def volume_up(self, secret, amount, zone = None):
        if self.invalid_secret(secret):
            return "FORBIDDEN"
        try:
            self.control.volume_up(amount, zone)
            return "OK"
        except:
            return "FAIL"

    def volume_down(self, secret, amount, zone = None):
        if self.invalid_secret(secret):
            return "FORBIDDEN"
        try:
            self.control.volume_down(amount, zone)
            return "OK"
        except:
            return "FAIL"

    def mute(self, secret, zone = None):
        if self.invalid_secret(secret):
            return "FORBIDDEN"
        try:
            self.control.mute(zone)
            return "OK"
        except:
            return "FAIL"

    def mute_on(self, secret, zone = None):
        if self.invalid_secret(secret):
            return "FORBIDDEN"
        try:
            self.control.mute_on(zone)
            return "OK"
        except:
            return "FAIL"

    def mute_off(self, secret , zone = None):
        if self.invalid_secret(secret):
            return "FORBIDDEN"
        try:
            self.control.mute_off(zone)
            return "OK"
        except:
            return "FAIL"
    def close_connection(self, secret):
        if self.invalid_secret(secret):
            return "FORBIDDEN"
        try:
            self.control.close_connection()
            return "OK"
        except:
            return "FAIL"

try:
    start = RemoteControl(control, server, secret)
except KeyboardInterrupt:
    print "Shutting down server..."
